/**********************************************************************
*example code how to use the hypothesis filter.
*
*using blob analysis
*
*
* @author: Manuel Boissenin
* @date: 20/05/2004
*
* Time-stamp: <2005-05-20 12:45:58 engmb>
* Copyright (C) 2004 by Manuel Boissenin
*
 ***********************************************************************/

#include <string>
#include <iostream>
#include <iomanip>
#include <sstream>
#include <fstream>
#include <GL/glut.h>
#include <image.h>
#include <image_fileinput.h>
#include <image_fileoutput.h>
#include <image_mesaoutput.h>

//!!change the name of p_condensation

#include "p_condensation.h"
#include "hypothesis2D.h"
#include "correlation_observer2D.h"
#include "analyse_result_blob.h"
#include "image_loader.h"
#include "compensate_speed.h"

using namespace mimas::hf;
using namespace std;

int main(int argc, char** argv)
{
  int retVal = 0;
  try {
    glutInit( &argc, argv );
    x11_display disp;    
    image_mesaoutput<unsigned char > win( &disp );
    
    //Initialisation of the tracker
    hypothesis2D init(atoi(argv[2]),atoi(argv[3]), 1.);
    hypothesis2D::sigmaSpan = atof(argv[4]);

    image_loader<unsigned char>::set il_param(argv[1], 99, ".png");
    correlation_observer2D<hypothesis2D, unsigned char>::set obs_param(atoi(argv[2]), atoi(argv[3]), 40, 40);
    analyse_result_blob<hypothesis2D, unsigned char>::set analyse_param(.55 ,10 ,7);
    
    hypothesis_filter<image_loader, correlation_observer2D, analyse_result_blob,
      hypothesis2D, p_condensation, no_compensation> hf(init, atoi(argv[5]), il_param, obs_param, analyse_param);

    //tracking
    int imageNumber = 0;
    
    while(true)
      {
	try{hf.next();}
	catch (mimasexception &e){
	  std::cerr<<e.what()<<std::endl;  
	  return 0;
	}

	std::vector<hypothesis2D> estimatedLocation;
	hf.filter();
	estimatedLocation = hf.analyse();
	
	hf.draw(estimatedLocation);//from Observer

	//----------save on a file--------//
	
	std::ostringstream str;
	str << "results/pipetteBlob" << std::setw(3) << std::setfill( '0' ) <<  imageNumber++
	    << ".ppm";
	std::ofstream filteredOut(str.str().c_str(), std::ios::binary); 
	filteredOut << hf.image;
	//---------------------------------//

	win << hf.image;//<< mimas::pause();
      }
	 
    
  } catch ( exception &e )
    {
      cerr << "An exception occurred: " <<  e.what() << endl;
      retVal = 1;
    };
  return retVal;
}
