/**The position of the object is its center expressed
in the image coordinate system
We are working here with feature images so we don't need to 
extract features of any images.
The origin position of the object is taken to be the top left corner 
of the template image.
*/  

#define QUIET_TESTS //this allows to do tests in the backgroung without having windows poping out on the foregrond.

#include "stencil_tracking.h"
#include <GL/glut.h>
#include <image_funcs.h>
#include <filter.h>
#include <fstream>
#include <image_fileinput.h>
#include <image_loader.h>
#include <primitives.h>
#include <string>


using namespace boost;
using namespace boost::numeric;
using namespace mimas;
using namespace std;

//typedef boost::numeric::ublas::vector<int> Feature;

//To extract the feature of the image
//and reproject them on the frame of the 
//previous location 

class object_position
{
public:
  int x;
  int y;
};

//We assume that images represent features by
//black pixels on a background of a different color.
std::vector<Feature>
get_features(string image_name)
{
  std::vector< Feature > features;
  image<unsigned char > image; 
  std::ifstream file(image_name.c_str(), std::ios::binary);
  file >> image;
  
   for(int x = 0; x < image.getWidth(); x++)
      for(int y = 0; y < image.getHeight(); y++)
	if(image.getPixel(x,y) == 0)
	  {
	    Feature temp(2);
	    temp(0) = x;
	    temp(1) = y;
	    features.push_back(temp);
	  }

return features;
}

std::vector<Feature>
get_features(const image<unsigned char> &image)
{
  std::vector< Feature > features;
  
   for(int x = 0; x < image.getWidth(); x++)
      for(int y = 0; y < image.getHeight(); y++)
	if(image.getPixel(x,y) == 0)
	  {
	    Feature temp(2);
	    temp(0) = x;
	    temp(1) = y;
	    features.push_back(temp);
	  }

return features;
}

void display_features(std::vector<Feature> f, x11_display *disp, int border = 3)
{
  image_mesaoutput<unsigned char > display(disp);
  image<unsigned char> image;
  //Find the size of the image:
  int w_min = f[0](0);
  int w_max = f[0](0);
  int h_min = f[0](1);
  int h_max = f[0](1);

  for(std::vector<Feature>::iterator it = f.begin(); it != f.end() ; it++)
    {
      if ( (*it)(0) < w_min)
	w_min = (*it)(0);
      if ( (*it)(0) > w_max)
	w_max = (*it)(0);
      if ( (*it)(1) < h_min)
	h_min = (*it)(1);
      if ( (*it)(1) > h_max)
	h_max = (*it)(1);
    }

  image.init(w_max - w_min + 2 * border, h_max - h_min + 2 * border);
  //write on the image the features

  for(std::vector<Feature>::iterator it = f.begin(); it != f.end() ; it++)
    {
      image.pixel((*it)(0) - w_min + border, (*it)(1) - h_min + border) = 255;
    }
   display << image<< mimas::wait(100);
}

int main(int argc, char** argv)
{
  try {
    glutInit( &argc, argv );
    x11_display disp;    
    image_mesaoutput<unsigned char > display( &disp );
    
    std::string log_name(argv[9]);
    log_name += "position.txt";
    std::wofstream textfile(log_name.c_str());

    //begin load parametres

    string model_image = argv[7];
    string base_name_image = argv[8];
    
    image<unsigned char> current_image;

    mimas::hf::image_loader<unsigned char> im_ld(current_image , base_name_image, 4,".png");
 
    object_position current_position;
    //when we generated the images we specify the initial position as beeing the center of the image
    //make it more generic latter !!!!!!!
    current_position.x = current_image.getWidth() / 2; 
    current_position.y = current_image.getHeight() / 2; 
    std::cerr << "Initial position: " << current_position.x <<" "<<current_position.y<<std::endl;   
      
    int x_min = atoi(argv[1]);
    int x_max = atoi(argv[2]);
    int x_sub = atoi(argv[3]);
  
    int y_min = atoi(argv[4]);
    int y_max = atoi(argv[5]);
    int y_sub = atoi(argv[6]);
  
    float ratio = atof(argv[10]);

    std::vector< Feature > model = get_features(model_image);

#ifndef NDEBUG
    display_features(model, &disp);
#endif
   
    stencil_tracking tracker(model, &disp,
			    x_min,x_max,x_sub,
			    y_min,y_max,y_sub,0,0,1,1,argv[9]);

    tracker.fill_image_space();
    tracker.fill_image_space_I(ratio);
    tracker.get_robustness();
    int frame_number = 0;

    //To reset the file between executions.
    std::stringstream file_name_timings;
    file_name_timings << argv[9] <<"tracking_times" << ".txt";
    std::ofstream ofile_timings(file_name_timings.str().c_str(),std::ios_base::trunc);
    ofile_timings.close();
    //end reset

    //to save the command line that has been executed
    std::stringstream file_name_command;
    file_name_command << argv[9] <<"command_line.txt";
    std::ofstream ofile_command(file_name_command.str().c_str(),std::ios_base::trunc);
    for(int i = 0; i < argc; i++)
      ofile_command <<argv[i]<<" ";
    ofile_command.close();
    //end command line saving


    while(true)
      {
	std::vector< Feature > current_features = get_features(current_image);

	//express features relatively to the current_position:
	for(std::vector<Feature>::iterator it = current_features.begin(); it != current_features.end(); it++)
	  {
	    (*it)(0) -= current_position.x; 
	    (*it)(1) -= current_position.y; 
	  }

#ifndef FAST_TESTS
	cerr<<"display features: "<<endl;
#endif
#ifndef NDEBUG
#ifndef QUIET_TESTS
       	display_features(current_features, &disp);
#endif
#endif
	//Transformation result1 = tracker.track(current_features);

	Transformation result = tracker.track_I(current_features);

	//std::cerr<<"Diference of tracked location :"<<  round(result.intervals[0].lower_bound) - round(result1.intervals[0].lower_bound) 
	// << ", "<< round(result.intervals[1].lower_bound) - round(result1.intervals[1].lower_bound)<<std::endl;

	float percentage = float(result.votes) / (float) model.size();
#ifndef NDEBUG
#ifndef FAST_TESTS
	cerr<<"Indicator of success is "<<percentage<<endl;
#endif
#endif
	/*
	current_position.x += (int)( (result.intervals[0].lower_bound + result.intervals[0].upper_bound) / 2.0 );
	current_position.y += (int)( (result.intervals[1].lower_bound + result.intervals[1].upper_bound) / 2.0 );
	*/

	//temporarily to test the algo precisely using steps of one pixel


	current_position.x += (int)( round(result.intervals[0].lower_bound));
	current_position.y += (int)( round(result.intervals[1].lower_bound));
#ifndef FAST_TESTS
	std::cerr << "object position" << current_position.x <<" "<<current_position.y  << std::endl;
#endif
	current_image.setPixel(current_position.x, current_position.y, 0);

	textfile<<frame_number++ <<" "<< current_position.x <<" "<<current_position.y<<std::endl;
#ifndef FAST_TESTS
#ifndef NDEBUG
	display << current_image << mimas::wait(100);
#endif
#endif
	try{
	im_ld.next();
	}
	catch (mimasexception &e)
	  {
	    std::cerr << "An exception occurred: " <<  e.what() << " probably because it was the last file"<<std::endl;
	    break;
	  }
      }
  } catch ( exception &e )
    {
      cerr << "An exception occurred: " <<  e.what() << endl;
      return EXIT_FAILURE;
    }
  std::cerr<<"exit success"<<std::endl;
  return EXIT_SUCCESS;
}

 

