/**the position of the object is its center expressed
in the image coordinate system
*/  
#include <GL/glut.h>
#include <image_funcs.h>
#include <filter.h>
#include <fstream>
#include <image_fileinput.h>
#include <image_fileoutput.h>
#include <image_loader.h>
#include <primitives.h>
#include "image_fileinput.h"
#include <stdlib.h>
#include <iostream>


using namespace mimas;
using namespace std;

//typedef boost::numeric::ublas::vector<int> Feature;

//To extract the feature of the image
//and reproject them on the frame of the 
//previous location 

class object_position
{
public:
  int x;
  int y;
  
  //  extract_features(const image<T>)
};

//parametres:image of features, number of image to generate, size of the image
//movement x and y drift

int main(int argc, char** argv)
{
  int retVal = 0;
  try {
    glutInit( &argc, argv );
    x11_display disp;    
    image_mesaoutput<unsigned char > display( &disp );

    image<unsigned char> template_image;
    image<unsigned char> image;

    int nb_images = atoi(argv[2]);
    int image_number = 0;

    int width = atoi(argv[3]); 
    int height = atoi(argv[4]);
    image.init(width,height);

    int x_drift = atoi(argv[5]);
    int y_drift = atoi(argv[6]);

    int false_positive_percentage = atoi(argv[7]);
    int false_negative_percentage = atoi(argv[8]);
    
    std::string output_basename = argv[9];

    int x_initial_position = width/2;
    int y_initial_position = height/2;

    //string template_name = string(argv[1]);
    ifstream file( argv[1], ios::binary );
    file >> template_image;

    //load the image of feature
    vector<point> features;
    for(int x = 0; x < template_image.getWidth(); x++)
      for(int y = 0; y < template_image.getHeight(); y++)
	if(template_image.getPixel(x,y) == 0)
	  features.push_back(point(x,y));
    
    int x_current_position = x_initial_position;
    int y_current_position = y_initial_position;
    int x_intermediate_position;
    int y_intermediate_position;


    std::string position_file_name = output_basename;
      position_file_name += "_position.txt";
    std::wofstream textfile(position_file_name.c_str());

    //Save first image without movement
    image.fill(255);
    for(vector<point>::iterator it = features.begin(); it != features.end(); it++)
      {
	image.setPixel(x_current_position + it->x, y_current_position + it->y, 0);
      }

    std::ostringstream str;
    textfile <<image_number << " " << x_current_position << " " << y_current_position << std::endl;
    str << output_basename << std::setw(4) << std::setfill( '0' ) <<  image_number++
	<< ".png";
    std::ofstream filteredOut(str.str().c_str(), std::ios::binary); 
    filteredOut << image;
    display<<image;


    for (int i = 0; i < nb_images; i++)
      {
	image.fill(255);

	do
	  {
	    x_intermediate_position = x_current_position + int(((double) rand()/ (double)RAND_MAX - 0.5) * x_drift);
	    y_intermediate_position = y_current_position + int(((double) rand()/ (double)RAND_MAX - 0.5) * y_drift);
	  }
	while(x_intermediate_position < -(template_image.getWidth()/2) || x_intermediate_position > (-template_image.getWidth()/2 + image.getWidth())||
	      y_intermediate_position < -(template_image.getHeight()/2) || y_intermediate_position > (-template_image.getHeight()/2 + image.getHeight()));

	x_current_position = x_intermediate_position;
	y_current_position = y_intermediate_position;

	for(vector<point>::iterator it = features.begin(); it != features.end(); it++)
	  {
	    image.setPixel(x_current_position + it->x, y_current_position + it->y, 0);
	  }

	//add noise (we add it only on the surroundingof the template.) the surrounding being 5 times the maximum distance 
	//where the template can be.

	int x_low = std::max(0, x_current_position - 5 * x_drift);
	int x_high = std::min(image.getWidth(), x_current_position + template_image.getWidth() + 5 * x_drift);
	int y_low = std::max(0, y_current_position - 5 * y_drift);
	int y_high =  std::min(image.getHeight(), y_current_position + template_image.getHeight() + 5 * y_drift);

	for (int x = x_low; x <x_high; x++)
	  for (int y = y_low; y <y_high; y++)
	    {
	      if (image.getPixel(x,y) == 0)
		{
		  if (100 * (double(random())/double(RAND_MAX)) < false_positive_percentage)
		    image.setPixel(x,y,255);
		}
	      else
		{
		  if (100 * (double(random())/double(RAND_MAX)) < false_negative_percentage)
		    image.setPixel(x,y,0);
		}

	    }

	textfile <<image_number << " " << x_current_position << " " << y_current_position << std::endl;

	std::ostringstream str;
	str << output_basename << std::setw(4) << std::setfill( '0' ) <<  image_number++
	    << ".png";
	std::ofstream filteredOut(str.str().c_str(), std::ios::binary); 
	filteredOut << image;
	      display<<image;

      }
    
  } catch ( exception &e )
    {
      cerr << "An exception occurred: " <<  e.what() << endl;
      retVal = 1;
    }
  return retVal;
}

 

