// You can only define one of these:
// #define TESTMESAOFFSCREEN

#include <cmath>
#include <fstream>
#include <functional>
#include <GL/glut.h>
#ifdef TESTMESAOFFSCREEN
#include "mesa_offscreen.h"
#include "timer.h"
#endif
#include "image_xineinput.h"
#include "image_mesaoutput.h"

using namespace boost;
using namespace std;
using namespace mimas;

typedef rgba< unsigned char > pixel_t;

int main( int argc, char *argv[] )
{
  int retVal = 0;

  try {

    // Note, that this is a very inefficient way to do it, if you just want
    // to display a movie. If you just want to implement a video player, you
    // should allow libxine do write directly to X11 memory!
    // See http://xinehq.de/index.php/hackersguide#AEN80 for more information.

    // Initialise GLUT.
    glutInit( &argc, argv );

    MMERROR( argc == 2, mimasexception, ,
             "Syntax: " << argv[0] << " <media resource location>" );

    // Open X11 display.
    x11_display display;

    const int delay = 100;

    image_mesaoutput< pixel_t > window( &display );
    image_xineinput< pixel_t > input( argv[1], "auto", "auto" );


#ifdef TESTMESAOFFSCREEN
    timer t;
#endif

    // Allocate image.
    image< pixel_t > img;

    // Loop for images.
    do {
      
      try {
        input >> img;

#ifdef TESTMESAOFFSCREEN
        mesa_offscreen< rgba< unsigned char > > m( &display, img, false );
        m.ortho2D();
        const rgba< unsigned char > red( 255, 0, 0 );
        m.setColor( red );

        glPushMatrix();
        double animate = cos( fmod( t.elapsed() * M_PI / 2.0, 2.0 * M_PI ) );
        glTranslated( 10, 10 + animate * 6, 0 );
        glScaled(  0.1,
		  -0.1 * animate,
		   1.0 );
        m.text( "Mimas" );
        glPopMatrix();
        m.finish();
#endif

        window << img << mimas::delay( delay );

      } catch ( exception &e ) {
        cerr << "Warning: " << e.what() << endl;
        window << mimas::wait( delay );
      };
      
    } while ( input );

  } catch ( exception &e ) {
    cerr << "Error: " << e.what() << endl;
    retVal = 1;
  };
  
  return retVal;
}
