//
// Method to find and track pipette tip using the Haynes and Jain edge-change
// detector and a recursive search algorithm. Note that this implementation
// assumes that the pipette is approaching from the top of the screen.
// 
// In order for initial recognition to take place the pipette has to be moved
// (ideally horizontally) so that the initial estimate of the tip can be
// obtained.
//
// Bala Amavasai (bala@amavasai.org)
// $Header: /cvs/mimas2/include/findpipette.h,v 1.1.1.1 2005/08/09 15:37:45 engmb Exp $
//

#ifndef FINDPIPETTE_H
#define FINDPIPETTE_H

#include <iostream>
#include <cmath>
#include <values.h>

#include "image.h"
#include "image_funcs.h"
#include "cartesian.h"

namespace mimas {
	
class findPipette
{
    private:
	int skipXboundaries; // left and right boundary to skip
	int skipYboundaries; // top and botton boundary to skip
	int ySearchRange;  // search from -ve in the y direction when not first frame
	int thresval;        // hj threshold value
	int linesDiffRange; // difference between longest and shortest line that is considered
	int holdPositionCondition; // hold previous position if outside this range
	bool firstrecog;    // recognition for the first time so that prevdepth can be set
	int scanliney;

	Cartesian<int> prevdepth, currdepth;
	void searchDown(int x, int y, int colour, image<int> &imagein, Cartesian<int> &d);

    protected:

    public:
	void find(image<int> &imageprev, image<int> &imagecurrent, int &x, int &y);
	void setSkipXboundaries(int tempval);
	void setSkipYboundaries(int tempval);
	int getSkipYboundaries(void);
	void setYSearchRange(int tempval);
	void setFirstRunFlag(bool tempval);
	void setHJThresholdVal(int tempval);
	void setLinesDiffRange(int tempval);
	void setHoldPositionCondition(int tempval);

 	findPipette()
	{
	    skipXboundaries=20;
	    skipYboundaries=10;
	    ySearchRange=20;
	    thresval=5;
	    linesDiffRange=20;
	    holdPositionCondition=30;
	    firstrecog=true;
	    scanliney=skipYboundaries;
	}

	~findPipette()
	{
	}
	
};
};

#endif
