namespace mimas {

template< typename T >
void image_fileinput< T >::read( image< T > &im ) throw (mimasexception)
{
  MMERROR( image_fileinput< T >::state, mimasexception, ,
           "Attempt to read image, after image_fileinput indicated end "
           "of file." );

  try {

    if ( !images ) {
      
      images = boost::shared_ptr< std::list< Magick::Image > >
        ( new std::list< Magick::Image > );
      
      std::streampos pos = stream->tellg();
      stream->seekg( 0, std::ios::end );
      int filesize = stream->tellg() - pos;
      stream->seekg( pos, std::ios::beg );

      MMERROR( *stream, mimasexception, , "Error accessing stream." );
      MMERROR( filesize > 0, mimasexception, , "Stream is empty." );

      char *data = new char[ filesize ];
      stream->read( data, filesize );

      Magick::Blob blob;
      blob.updateNoCopy( data, filesize, Magick::Blob::NewAllocator );
      readImages( images.get(), blob );

      if ( images->empty() ) {
        // Usually the file was not an image.
        mimasexception e;
        e << "Stream didn't yield any image (readable formats are:";
        std::map< std::string, bool > fileFormats
          ( image_fileinput< T >::formats() );
        for ( std::map< std::string, bool >::const_iterator i =
                fileFormats.begin(); i != fileFormats.end(); i++ )
          e << ' ' << i->first;
        e << ").";
        throw e;
      };
    };

    assert( images );
    assert( !images->empty() );

    std::list< Magick::Image >::iterator i = images->begin();
    
    im.init( i->columns(), i->rows() );
    // Direct access to image-memory!
    i->write( 0, 0, i->columns(), i->rows(), mapString(), Magick::CharPixel,
              im.rawData() );

    images->pop_front();

  } catch ( std::exception &e ) {
    
    MMERROR( false, mimasexception, ,
             "Error reading image from stream: " << e.what() );
    
  };

  image_fileinput< T >::state = !images->empty();
}

template< typename T >
std::map< std::string, bool > image_fileinput< T >::formats(void)
{
  std::map< std::string, bool > retVal;
  std::list< Magick::CoderInfo > coderList;
  coderInfoList( &coderList, Magick::CoderInfo::TrueMatch,
                 Magick::CoderInfo::AnyMatch,
                 Magick::CoderInfo::AnyMatch );
  for ( std::list< Magick::CoderInfo >::iterator i = coderList.begin();
        i != coderList.end(); i++ )
    retVal[ i->name() ] = i->isMultiFrame();
  return retVal;
}

};
