namespace mimas {

template< typename T >
void image_fileoutput< T >::write( const const_image_ref< T > &img )
{
  // Default file type is pgm.
  if ( image_fileoutput_fileType == NULL )
    image_fileoutput_fileType = defaultFormat();

  const char *currentFormat = image_fileoutput_fileType;

  // Reset output format to avoid side-effects.
  image_fileoutput_fileType = NULL;

  if ( buffer.size() >= 1 ) {
    // Check, wether writing another image is supported.
    MMERROR( image_fileoutput< T >::state, mimas::mimasexception, ,
             "Attempt to store multiple images in single-image file format \""
             << buffer.front().magick() << "\"." );
  };

  Magick::Image magickImage;
  buffer.push_back( magickImage );
  buffer.back().read( img.getWidth(), img.getHeight(), mapString(),
                      Magick::CharPixel, img.rawData() );

  if ( buffer.size() > 1 ) {

    buffer.back().magick( buffer.front().magick() );
    buffer.back().compressType( buffer.front().compressType() );
    
  
  } else {
    
    assert( currentFormat != NULL );
    
    std::map< std::string, bool > fileFormats
      ( image_fileoutput< T >::formats() );
    std::map< std::string, bool >::const_iterator entry =
      fileFormats.find( currentFormat );
    
    if ( entry == fileFormats.end() ) {
      // The specified file-format is not supported.
      mimasexception e;
      e << "Unknown file-format \"" << currentFormat
        << "\" (writable formats are:";
      for ( std::map< std::string, bool >::const_iterator i =
              fileFormats.begin(); i != fileFormats.end(); i++ )
        e << ' ' << i->first;
      e << ").";
      throw e;
    } else
      // Check, wether writing of multiple images is allowed.
      image_fileoutput< T >::state = entry->second;
    
    buffer.back().magick( currentFormat );

    // Use binary version of format!
    buffer.back().compressType( Magick::UndefinedCompression );

  };

  // Indicate errors on associated stream?
  // stream->setstate( std::ios_base::failbit );
}

template< typename T >
std::map< std::string, bool > image_fileoutput< T >::formats(void)
{
  std::map< std::string, bool > retVal;
  std::list< Magick::CoderInfo > coderList;
  coderInfoList( &coderList, Magick::CoderInfo::AnyMatch,
                 Magick::CoderInfo::TrueMatch,
                 Magick::CoderInfo::AnyMatch );
  for ( std::list< Magick::CoderInfo >::iterator i = coderList.begin();
        i != coderList.end(); i++ )
    retVal[ i->name() ] = i->isMultiFrame();
  return retVal;
}

template< typename T >
image_fileoutput< T >::~image_fileoutput(void)
{
  Magick::Blob blob;
  Magick::writeImages( buffer.begin(), buffer.end(), &blob, true );
  stream->write( (const char *)blob.data(), blob.length() );
} 

};
