#ifndef X11_DISPLAY_H
#define X11_DISPLAY_H

#include "mimasconfig.h"

#ifdef HAVE_OPENGL

#include <boost/shared_ptr.hpp>
#include <climits>
#include <map>
#include <string>
#include <X11/Xlib.h>
#include "mimasexception.h"
#include "x11_window.h"

namespace mimas {

/** Wrapper for X11-display.
    This wrapper provides automatic destruction for an X11-display-variable.
    @author Jan Wedekind (jan at wedesoft.de)
    @date Fri Apr 08 10:07:26 UTC 2005 */
class x11_display: public object
{
 public:
  /** Constructor.
      Open X11-display.
      @param name Name of X11-display to use (default: use content of
      <TT>DISPLAY</TT>-variable. */
  x11_display( const std::string &name = "" )
    throw (mimasexception);
  /// Destructor.
  virtual ~x11_display(void) { XCloseDisplay( display ); }
  /// Get pointer to X11-display.
  Display *get(void) { return display; }
  /** Loop for events.
      Loop for events until \c quit() is called by an \c x11_window object or
      until a time-out occurs.
      @param mask Mask for filtering events.
      @param timeout Time limit for the event-loop in milliseconds. Specify
      \c INT_MAX for no limit (default).
      @see process_events */
  void event_loop( int timeout = INT_MAX,
		   long mask = KeyPressMask |
		   ExposureMask |
		   StructureNotifyMask )
    throw (mimasexception);
  /** Process pending events.
      @see event_loop */
  void process_events( long mask = KeyPressMask |
		       ExposureMask |
		       StructureNotifyMask )
    throw (mimasexception);
  ///
  void add_window( x11_window *win );
  ///
  void remove_window( x11_window *win );
  ///
  void quit(void) { doQuit = true; }
 protected:
  ///
  void handle_event( XEvent &event ) throw (mimasexception);
  /// X11 display-variable.
  Display *display;
  /// Map of event-handlers.
  std::map< Window, x11_window * > windows;
  ///
  bool doQuit;
};

///
typedef boost::shared_ptr< x11_display > x11_display_ptr;

};

#endif

#endif

