//
// stuart meikle Fri Jun  9 10:10:30 2000
//
// hough transform functions
//
// $Header: /cvs/mimas2/lib/hough_transform.cc,v 1.1.1.1 2005/08/09 15:37:29 engmb Exp $

#include <cmath>
#include "functions.h"
#include "hough_transform.h"

using namespace boost;
using namespace std;

namespace mimas {

//private methods
int	hough_transform::calcGaussBoxSize( double sigma, double accuracy )
{
    //code largely taken from tina src.
    double n;
    
    sigma = 2.0 * sigma * sigma;
    
    for(n=1.0; ;n+=1.0)
    {
	if (exp( -n*n/sigma )< accuracy) break;
    }
    
    return (int) n;
}


//public services
void	hough_transform::addGaussPoint( const IntVector &posn, double weight, double sigma, double precision )
{
    //NOTE: when constructing pghs we don't use this function we use the traditional tina ones.
    //dunno why, but i suspect they're faster. 

    //first order approximation of course!
    //quite a poor algorithm here (@) could be speeded up a great deal
    int	size;
    int	lx,ly,ux,uy;
    int x,y;
    double sig;
    
    sig = 2.0 *sigma * sigma;
    
    size = calcGaussBoxSize( sigma, precision );
    lx = (int) posn(0) - size;
    if (lx<0)	lx = 0;

    ux = (int) posn(0) + size;
    if (ux>=getWidth())	ux=getWidth()-1;

    ly = (int) posn(1) - size;
    if (ly<0)	ly = 0;

    uy = (int) posn(1) + size;
    if (uy>=getHeight()) uy=getHeight()-1;
    
    for(y=ly;y<=uy;y++)
    {
	for(x=lx;x<=ux;x++)
	{
	    double	dx,dy,d,w;
	    
	    dx = posn(0) - x;
	    dy = posn(1) - y;
	    d  = dx*dx + dy*dy;
	    
	    w  = exp ( -d/sig ) *weight;
	    
	    setPixel( x, y, getPixel( x, y ) + w );
	}
    }
}


void	hough_transform::addGaussLine( line line, double weight, double sigma, double precision )
{

    Vector v( line.getVector() );
    double	x0, y0;
    double	sin_theta, cos_theta;
    double	divisor;
    double	theta;
    int		x,y;
    double	size,l;
    double	xstep,ystep;
    double	sig,w;
    
    sig = 2.0 * sigma * sigma;
    
    //standard co-ordinate transform. imagine line goes from (0,0) - (length,0) in x' y' space
    theta = atan2( v(1), v(0) );
    sin_theta = sin(theta);
    cos_theta = cos(theta);
    divisor = sin_theta*sin_theta + cos_theta*cos_theta;
    size = (double) calcGaussBoxSize( sigma, precision );
    l = line.length();
    x0 = line.getStart()(0);
    y0 = line.getStart()(1);
    xstep = cos_theta/divisor;		// ( divisor is never =0 )
    ystep = -sin_theta/divisor;
    
    //!! WARNING: major performance improvements could be made here!!
    
    for(y=0;y<getHeight();y++)
    {
	double	xdash, ydash;

        xdash = ((double)y-y0)*sin_theta + ((double) 0 - x0)*cos_theta;
        xdash/=divisor;
	    
        ydash = ((double)y-y0)*cos_theta - ((double) 0 - x0)*sin_theta;
        ydash/=divisor;
	

	for(x=0;x<getWidth();x++)
	{
	    //calculate the perpendicular distance from (i,j) to the line
	    //should be able to transform i, j directly to the new co-ordinate system
	    
	    if (xdash>-size && xdash< (l+size) && ydash<size && ydash>-size)
	    {
		//plot a pixel (bounds checking done automatically here)
		double	dx,dy,d;
		
		if (xdash>0 && xdash<l)
		{
		    dx =0;	dy=ydash;
		}
		else
		if (xdash<0)
		{
		    dx = xdash;	dy=ydash;
		}
		else
		if (xdash>l)
		{
		    dx = xdash-l; dy=ydash;
		}
		
		d = dx*dx + dy*dy;
		w  = exp ( -d/sig ) *weight;
		pixel( x,y ) += w;
	    }
	    
	    //use the derivative to speed things up!
	    xdash += xstep;
	    ydash += ystep;
	}
    }
}

void	hough_transform::addGaussLineFast( line line, double weight )
{
//you must call initFastGauss first!

         IntVector p( 2 );
			int i,dx,dy,sdx,sdy,dxabs,dyabs,x,y;
			int x1,x2,y1,y2;

			x1 = (int)line.getStart()(0);
			x2 = (int)line.getEnd()(0);			
			y1 = (int)line.getStart()(1);
			y2 = (int)line.getEnd()(1);

			dx=x2-x1;			/* the horizontal distance of the line */
			dy=y2-y1;			/* the vertical distance of the line */
			dxabs=abs(dx);
			dyabs=abs(dy);
			sdx=sgn(dx);
			sdy=sgn(dy);
			x=dyabs>>1;
			y=dxabs>>1;
			p(0) = x1;
			p(1) = y1;

			addGaussPointFast( p );

			if (dxabs>=dyabs) /* the line is more horizontal than vertical */
			{
				for(i=0;i<dxabs;i++)
				{
					y+=dyabs;
					if (y>=dxabs)
					{
						y-=dxabs;
						p(1) += sdy;
					}
					p(0) += sdx;
					addGaussPointFast( p );					
				}
			}
			else /* the line is more vertical than horizontal */
			{
				for(i=0;i<dyabs;i++)
				{
					x+=dxabs;
					if (x>=dyabs)
					{
						x-=dyabs;
						p(0) += sdx;
					}
					p(1) += sdy;
					addGaussPointFast( p );


				}
			}
}


void	hough_transform::addSlopedGaussLine( line line, double weight, double sigma, double precision )
{
    //as for add guass line, but with varying weight along the length. linear from 0 to weight
    //and back to zero.
    // trial code by stuM Tue Jun 20 09:50:46 2000


    Vector v( line.getVector() );
    double	x0, y0;
    double	sin_theta, cos_theta;
    double	divisor;
    double	theta;
    int		x,y;
    double	size,l;
    double	xstep,ystep;
    double	sig,w;
    double	newweight;
    
    sig = 2.0 * sigma * sigma;
    
    //standard co-ordinate transform. imagine line goes from (0,0) - (length,0) in x' y' space
    theta = atan2( v(1), v(0) );
    sin_theta = sin(theta);
    cos_theta = cos(theta);
    divisor = sin_theta*sin_theta + cos_theta*cos_theta;
    size = (double) calcGaussBoxSize( sigma, precision );
    l = line.length();
    x0 = line.getStart()(0);
    y0 = line.getStart()(1);
    xstep = cos_theta/divisor;		// ( divisor is never =0 )
    ystep = -sin_theta/divisor;
    
    for(y=0;y<getHeight();y++)
    {
	double	xdash, ydash;

        xdash = ((double)y-y0)*sin_theta + ((double) 0 - x0)*cos_theta;
        xdash/=divisor;
	    
        ydash = ((double)y-y0)*cos_theta - ((double) 0 - x0)*sin_theta;
        ydash/=divisor;
	

	for(x=0;x<getWidth();x++)
	{
	    //calculate the perpendicular distance from (i,j) to the line
	    //should be able to transform i, j directly to the new co-ordinate system
	    if (xdash< (l/2.0))
		newweight = xdash / (l/2.0) * weight;
	    else
		newweight = (xdash - l)/(l/2.0) * weight;
		
	    if (newweight <0)	newweight =0;	    
	    
	    if (xdash>-size && xdash< (l+size) && ydash<size && ydash>-size)
	    {
		//plot a pixel (bounds checking done automatically here)
		double	dx,dy,d;
		
		if (xdash>0 && xdash<l)
		{
		    dx =0;	dy=ydash;
		}
		else
		if (xdash<0)
		{
		    dx = xdash;	dy=ydash;
		}
		else
		if (xdash>l)
		{
		    dx = xdash-l; dy=ydash;
		}
		
		d = dx*dx + dy*dy;
		w  = exp ( -d/sig ) *newweight;
		pixel( x,y ) += w;
	    }
	    
	    //use the derivative to speed things up!
	    xdash += xstep;
	    ydash += ystep;
	}
    }
}






void	hough_transform::normalize(void)
{
    double	sum;
    int		i,j;
    
    sum=0;
    for(j=0;j<getHeight();j++)
    {
	for(i=0;i<getWidth();i++)
	{
	    sum += fabs(pixel(i,j));
	}
    }

    if (sum<0.0001)	return;
    for(j=0;j<getHeight();j++)
    {
	for(i=0;i<getWidth();i++)
	{
     pixel(i,j) /= sum;
	}
    }
}



bool	hough_transform::lineHitsPoint( const IntVector &point, line line, 
                                         double sigma, double precision )
{
    //does the blurred line pass through the point `v`? return true if yes.
    Vector v( line.getVector() );
    double	x0, y0;
    double	sin_theta, cos_theta;
    double	divisor;
    double	theta;
    double	x,y;
    double	size,l;
    double	sig;
    
    sig = 2.0 * sigma * sigma;
    
    //standard co-ordinate transform. imagine line goes from (0,0) - (length,0) in x' y' space
    theta = atan2( v(1), v(0) );
    sin_theta = sin(theta);
    cos_theta = cos(theta);
    divisor = sin_theta*sin_theta + cos_theta*cos_theta;
    size = (double) calcGaussBoxSize( sigma, precision );
    l = line.length();
    x0 = line.getStart()(0);
    y0 = line.getStart()(1);
    
    y = point(1);
    x = point(0);

    double	xdash, ydash;	

    xdash = (y-y0)*sin_theta + (x - x0)*cos_theta;
    xdash/=divisor;
	    
    ydash = (y-y0)*cos_theta - (x - x0)*sin_theta;
    ydash/=divisor;
	
    if (xdash>-size && xdash< (l+size) && ydash<size && ydash>-size)
    {
	return true;
    }
    else
	return false;
}


//------------ ! these next are strictly speaking `bad' because they assume we 
// have a hough transform which has the dimensions 360 x 1

void	hough_transform::makeAngleEntry(angle a, double sigma, double prec )
{
    IntVector p( 2 );
    p( 0 ) = (int)( ( (double)a + M_PI ) * 180.0 / M_PI );
	    
    //addGaussPoint( vector( x, 5.5 ) , 10.0, 30.0, 0.01 );
    //addGaussLine( line( x,0,x,10 ), 1.0 , sigma, 0.01 );
    addGaussPoint( p, 1.0, sigma * 180.0 / M_PI, 0.01 );
    p( 0 ) -= 360;
    addGaussPoint( p, 1.0, sigma * 180.0 / M_PI, 0.01 );
    //addGaussLine ( line( x,0,x,10 ), 200.0, 3.0, 0.01 );
}



angle 	hough_transform::getAngle( void )
{
    double	max_v=0;
    double	a;
    int		max_x=0;
    int		i=0;
    
    for(i=0;i<360;i++)
	if (getPixel(i,0) > max_v)
	{
	    max_v = getPixel(i,0);
	    max_x = i;
	}
    
    a= (double) max_x - 180;
    a=a*0.0174533;
    
    return angle(a);
    
}

//-------------------- routines for faster point plotting --------------------
//stumeikle Sun May 26 11:31:40 2002
void	hough_transform::initFastGauss( double sigma, double precision )
{
    // create a convolution-like kernal for a gauss blob

    // if (bell_curve)	delete bell_curve;

    int	size;
    double sig;
    
    sig = 2.0 *sigma * sigma;
    size = calcGaussBoxSize( sigma, precision );
    
    bell_curve = shared_ptr< image< double > >( new image< double > );
    bell_curve->init( size*2, size*2 );
    
    for(int y=0;y<(size*2);++y)
    {
	for(int x=0;x<(size*2);++x)
	{
	    double dx,dy,d,w;
	    
	    dx = (double)(x-size);
	    dy = (double)(y-size);
	    d  = dx*dx+dy*dy;
	    
	    w  = exp ( -d/sig );
	    bell_curve->pixel(x,y) = w;
	}
    }
}


void	hough_transform::addGaussPointFast( const IntVector &posn )
{
    //as for plot point, but the bell curve must be initialised first!
    int	size;
    int x,y,sx, sy,o_sx,o_sy;
    double v;

    if (!bell_curve)	return;

    size = bell_curve->getWidth();

    o_sx = posn(0) - size/2;
    o_sy = posn(1) - size/2;

    for(y=0, sy=o_sy;y<size;++y, ++sy)
    {
	for(x=0, sx=o_sx;x<size;++x, ++sx)
	{
	    //v=bell_curve->pixel(x,y);//*weight;
	    //setPixel(sx,sy,getPixel(sx,sy)+v);
	    v=bell_curve->getPixel(x,y);// *weight;
	    setPixel( sx, sy, getPixel(sx,sy) + v );
	}
    }

}

}
