#include "colourspace.h"
#include "image_dc1394input.h"
#include <string.h>

#ifdef HAVE_LIBDC1394
#include "rgba.h"

namespace mimas {

template<>
void image_dc1394input< unsigned char >::
read( image< unsigned char > &img ) throw (mimasexception)
{
  int width, height, size;

  switch ( mode ) {
  case MODE_640x480_MONO:
    width = 640;
    height = 480;
    size = 640 * 480;
    break;
  default:
    MMERROR( false, mimasexception, ,
             "Video-mode number " << mode << " not supported "
             "by mimas. Please contact the developers." );
  };

  img.init( width, height );

  // Perform image capture.
  MMERROR( dc1394_dma_single_capture( &camera ) == DC1394_SUCCESS,
           mimasexception, ,
           "Failed capturing frame from firewire digital camera." );

  memcpy( img.rawData(), camera.capture_buffer, size );

  // Allow buffer to be reused.
  dc1394_dma_done_with_buffer( &camera );
}

template<>
void image_dc1394input< rgba< unsigned char > >::
read( image< rgba< unsigned char > > &img ) throw (mimasexception)
{
  int width, height;

  switch ( mode ) {
  case MODE_320x240_YUV422:
    width = 320;
    height = 240;
    break;
  case MODE_640x480_YUV422:
    width = 640;
    height = 480;
    break;
  default:
    MMERROR( false, mimasexception, ,
             "Video-mode number " << mode << " not supported "
             "by mimas. Please contact the developers." );
  };

  // Initialise image.
  img.init( width, height );

  // Perform image capture.
  MMERROR( dc1394_dma_single_capture( &camera ) == DC1394_SUCCESS,
           mimasexception, ,
           "Failed capturing frame from firewire digital camera." );

  uyvy_to_rgba( (const char *)camera.capture_buffer, width, height,
		(char *)img.rawData() );

  // Allow buffer to be reused.
  dc1394_dma_done_with_buffer( &camera );
}

};

#endif
